@@ -66,6 +66,26 @@ Compiler::~Compiler()
 {
 }
 
+void
+Compiler::parseACX(string const &fichero, wstring const &dir)
+{
+  if(dir == COMPILER_RESTRICTION_LR_VAL)
+  {
+    reader = xmlReaderForFile(fichero.c_str(), NULL, 0);
+    if(reader == NULL)
+    {
+      cerr << "Error: cannot open '" << fichero << "'." << endl;
+      exit(EXIT_FAILURE);
+    }
+    int ret = xmlTextReaderRead(reader);
+    while(ret == 1)
+    {
+      procNodeACX();
+      ret = xmlTextReaderRead(reader);
+    }
+  }
+}
+
 void
 Compiler::parse(string const &fichero, wstring const &dir)
 {
@@ -86,7 +106,7 @@ Compiler::parse(string const &fichero, wstring const &dir)
 
   if(ret != 0)
   {
-    cerr << "Error: Parse error at the end of input." << endl;
+    wcerr << L"Error: Parse error at the end of input." << endl;
   }
 
   xmlFreeTextReader(reader);
@@ -118,8 +138,8 @@ Compiler::procAlphabet()
     }
     else
     {
-      cerr << "Error (" << xmlTextReaderGetParserLineNumber(reader);
-      cerr << "): Missing alphabet symbols." << endl;
+      wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
+      wcerr << L"): Missing alphabet symbols." << endl;
       exit(EXIT_FAILURE);
     }
   }
@@ -180,10 +200,15 @@ Compiler::matchTransduction(list<int> const &pi,
   }
   else
   {
+    map<int, set<int> >::iterator acx_map_ptr;
+    int rsymbol = 0;
+
     while(true)
     {
       int etiqueta;
-
+      
+      acx_map_ptr = acx_map.end();
+      
       if(izqda == limizqda && dcha == limdcha)
       {
         break;
@@ -196,16 +221,30 @@ Compiler::matchTransduction(list<int> const &pi,
       else if(dcha == limdcha)
       {
         etiqueta = alphabet(*izqda, 0);
+        acx_map_ptr = acx_map.find(*izqda);
+        rsymbol = 0;
         izqda++;
       }
       else
       {
         etiqueta = alphabet(*izqda, *dcha);
+        acx_map_ptr = acx_map.find(*izqda);
+        rsymbol = *dcha;
         izqda++;
         dcha++;
       }
 
-      estado = t.insertSingleTransduction(etiqueta, estado);
+      int nuevo_estado = t.insertSingleTransduction(etiqueta, estado);
+      
+      if(acx_map_ptr != acx_map.end())
+      {
+        for(set<int>::iterator it = acx_map_ptr->second.begin(); 
+            it != acx_map_ptr->second.end(); it++)
+        { 
+          t.linkStates(estado, nuevo_estado, alphabet(*it ,rsymbol));
+        }
+      }
+      estado = nuevo_estado;
     }
   }
 
@@ -304,8 +343,8 @@ Compiler::skipBlanks(wstring &name)
     {
       if(!allBlanks())
       {
-        cerr << "Error (" << xmlTextReaderGetParserLineNumber(reader); 
-        cerr << "): Invalid construction." << endl;
+        wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader); 
+        wcerr << L"): Invalid construction." << endl;
         exit(EXIT_FAILURE);
       }
     }
@@ -327,8 +366,8 @@ Compiler::skip(wstring &name, wstring const &elem)
     {
       if(!allBlanks())
       {
-        cerr << "Error (" << xmlTextReaderGetParserLineNumber(reader);
-        cerr << "): Invalid construction." << endl;
+        wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
+        wcerr << L"): Invalid construction." << endl;
         exit(EXIT_FAILURE);
       }
     }
@@ -468,8 +507,8 @@ Compiler::insertEntryTokens(vector<EntryToken> const &elements)
       }
       else
       {
-        cerr << "Error (" << xmlTextReaderGetParserLineNumber(reader);
-        cerr << "): Invalid entry token." << endl;
+        wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
+        wcerr << L"): Invalid entry token." << endl;
         exit(EXIT_FAILURE);
       }
     }
@@ -603,8 +642,8 @@ Compiler::procEntry()
     int ret = xmlTextReaderRead(reader);
     if(ret != 1)
     {
-      cerr << "Error (" << xmlTextReaderGetParserLineNumber(reader);
-      cerr << "): Parse error." << endl;
+      cerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
+      cerr << L"): Parse error." << endl;
       exit(EXIT_FAILURE);
     }
     wstring name = XMLParseUtil::towstring(xmlTextReaderConstName(reader));
@@ -669,6 +708,39 @@ Compiler::procEntry()
   }
 }
 
+void
+Compiler::procNodeACX()
+{
+  xmlChar  const *xnombre = xmlTextReaderConstName(reader);
+  wstring nombre = XMLParseUtil::towstring(xnombre);
+  if(nombre == L"#text")
+  {
+    /* ignore */
+  }
+  else if(nombre == L"analysis-chars")
+  {
+    /* ignore */
+  }
+  else if(nombre == L"char")
+  {
+    acx_current_char = static_cast<int>(attrib(L"value")[0]);
+  }
+  else if(nombre == L"equiv-char")
+  {
+    acx_map[acx_current_char].insert(static_cast<int>(attrib(L"value")[0]));
+  }
+  else if(nombre == L"#comment")
+  {
+    /* ignore */
+  }
+  else
+  {
+    wcerr << L"Error in ACX file (" << xmlTextReaderGetParserLineNumber(reader);
+    wcerr << L"): Invalid node '<" << nombre << L">'." << endl;
+    exit(EXIT_FAILURE);
+  }
+}
+
 void
 Compiler::procNode()
 {
