@@ -177,8 +177,8 @@ Expander::skip(wstring &name, wstring const &elem)
   {
     if(!allBlanks())
     {
-      cerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
-      cerr << L"): Invalid construction." << endl;
+      wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
+      wcerr << L"): Invalid construction." << endl;
       exit(EXIT_FAILURE);
     }
     xmlTextReaderRead(reader);
@@ -296,6 +296,25 @@ Expander::procEntry(FILE *output)
   wstring atributo=this->attrib(Compiler::COMPILER_RESTRICTION_ATTR);
   wstring entrname=this->attrib(Compiler::COMPILER_LEMMA_ATTR);
   
+  wstring myname = L"";
+  if(this->attrib(Compiler::COMPILER_IGNORE_ATTR) == L"yes")
+  {    
+    do
+    {
+      int ret = xmlTextReaderRead(reader);
+      if(ret != 1)
+      {
+        wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
+        wcerr << L"): Parse error." << endl;
+        exit(EXIT_FAILURE);
+      }
+      myname = XMLParseUtil::towstring(xmlTextReaderConstName(reader));
+     // wcerr << L"Hola " << myname << L" " << Compiler::COMPILER_ENTRY_ELEM << endl;
+    }
+    while(myname != Compiler::COMPILER_ENTRY_ELEM);
+    return;
+  }
+  
   EntList items, items_lr, items_rl;
   if(atributo == Compiler::COMPILER_RESTRICTION_LR_VAL)
   {
@@ -315,8 +334,8 @@ Expander::procEntry(FILE *output)
     int ret = xmlTextReaderRead(reader);
     if(ret != 1)
     {
-      cerr << "Error (" << xmlTextReaderGetParserLineNumber(reader);
-      cerr << "): Parse error." << endl;
+      wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
+      wcerr << L"): Parse error." << endl;
       exit(EXIT_FAILURE);
     }
     wstring name = XMLParseUtil::towstring(xmlTextReaderConstName(reader));
