@@ -18,9 +18,12 @@
  */
 #include <lttoolbox/fst_processor.h>
 #include <lttoolbox/compression.h>
+#include <lttoolbox/exception.h>
 
 #include <iostream>
 
+#include <errno.h>
+
 using namespace std;
 
 	
@@ -38,6 +41,7 @@ FSTProcessor::FSTProcessor()
   escaped_chars.insert(L'>');
   
   caseSensitive = false;
+  nullFlush = false;
 }
 
 FSTProcessor::~FSTProcessor()
@@ -47,8 +51,7 @@ FSTProcessor::~FSTProcessor()
 void
 FSTProcessor::streamError()
 {
-  cerr << "Error: Malformed input stream." << endl;
-  exit(EXIT_FAILURE);
+  throw Exception("Error: Malformed input stream.");
 }
 
 wchar_t
@@ -145,6 +148,78 @@ FSTProcessor::readAnalysis(FILE *input)
   return val;
 }
 
+int
+FSTProcessor::readTMAnalysis(FILE *input)
+{
+  isLastBlankTM = false;
+  if(!input_buffer.isEmpty())
+  {
+    return input_buffer.next();
+  }
+
+  wchar_t val = static_cast<wchar_t>(fgetwc_unlocked(input));
+  int altval = 0;
+  if(feof(input))
+  {
+    return 0;
+  }
+
+  if(escaped_chars.find(val) != escaped_chars.end() || iswdigit(val))
+  {
+    switch(val)
+    {
+      case L'<':
+        altval = static_cast<int>(alphabet(readFullBlock(input, L'<', L'>')));
+	input_buffer.add(altval);
+        return altval;
+
+      case L'[':
+        blankqueue.push(readFullBlock(input, L'[', L']'));
+        input_buffer.add(static_cast<int>(L' '));
+        isLastBlankTM = true;
+        return static_cast<int>(L' ');
+        
+      case L'\\':
+        val = static_cast<wchar_t>(fgetwc_unlocked(input));
+        if(escaped_chars.find(val) == escaped_chars.end())
+        {
+          streamError();
+        }
+        input_buffer.add(static_cast<int>(val));
+        return val;
+      case L'0':
+      case L'1':
+      case L'2':
+      case L'3':
+      case L'4':
+      case L'5':
+      case L'6':
+      case L'7':
+      case L'8':
+      case L'9':
+        {
+          wstring ws = L"";
+          do
+          {
+            ws += val;
+            val = static_cast<wchar_t>(fgetwc_unlocked(input));
+          } while(iswdigit(val));
+          ungetwc(val, input);
+          input_buffer.add(alphabet(L"<n>"));
+          numbers.push_back(ws);
+          return alphabet(L"<n>");
+        }
+        break;
+
+      default:
+        streamError();
+    }                   
+  }
+
+  input_buffer.add(val);
+  return val;
+}
+
 int
 FSTProcessor::readPostgeneration(FILE *input)
 {
@@ -366,10 +441,15 @@ FSTProcessor::classifyFinals()
       postblank.insert(it->second.getFinals().begin(),
                        it->second.getFinals().end());
     }
+    else if(endsWith(it->first, L"@preblank"))
+    {
+      preblank.insert(it->second.getFinals().begin(),
+                       it->second.getFinals().end());
+    }
     else
     {
-      wcerr << "Error: Unsupported transducer type for '";
-      wcerr << it->first << "'." << endl;
+      wcerr << L"Error: Unsupported transducer type for '";
+      wcerr << it->first << L"'." << endl;
       exit(EXIT_FAILURE);
     }
   }
@@ -487,6 +567,21 @@ FSTProcessor::initAnalysis()
   all_finals = standard;
   all_finals.insert(inconditional.begin(), inconditional.end());
   all_finals.insert(postblank.begin(), postblank.end());
+  all_finals.insert(preblank.begin(), preblank.end());
+}
+
+void
+FSTProcessor::initTMAnalysis()
+{
+  calcInitial();
+
+  for(map<wstring, TransExe, Ltstr>::iterator it = transducers.begin(),
+                                             limit = transducers.end();
+      it != limit; it++)
+  {
+    all_finals.insert(it->second.getFinals().begin(), 
+                      it->second.getFinals().end());
+  }
 }
 
 void
@@ -517,8 +612,14 @@ FSTProcessor::initBiltrans()
 void
 FSTProcessor::analysis(FILE *input, FILE *output)
 {
+  if(getNullFlush())
+  {
+    analysis_wrapper_null_flush(input, output);
+  }
+
   bool last_incond = false;
   bool last_postblank = false;
+  bool last_preblank = false;
   State current_state = initial_state;
   wstring lf = L"";
   wstring sf = L"";
@@ -551,6 +652,17 @@ FSTProcessor::analysis(FILE *input, FILE *output)
         last_postblank = true;
         last = input_buffer.getPos();      
       }
+      else if(current_state.isFinal(preblank))
+      {
+        bool firstupper = iswupper(sf[0]);
+        bool uppercase = firstupper && iswupper(sf[sf.size()-1]);
+
+        lf = current_state.filterFinals(all_finals, alphabet,
+                                        escaped_chars,
+                                        uppercase, firstupper);
+        last_preblank = true;
+        last = input_buffer.getPos();      
+      }
       else if(!isAlphabetic(val))
       {
         bool firstupper = iswupper(sf[0]);
@@ -560,6 +672,7 @@ FSTProcessor::analysis(FILE *input, FILE *output)
                                         escaped_chars, 
                                         uppercase, firstupper);
         last_postblank = false;
+        last_preblank = false;
         last_incond = false;
         last = input_buffer.getPos();
       }
@@ -569,6 +682,7 @@ FSTProcessor::analysis(FILE *input, FILE *output)
       lf = L"/*";
       lf.append(sf);
       last_postblank = false;
+      last_preblank = false;
       last_incond = false;
       last = input_buffer.getPos();
     }
@@ -603,13 +717,6 @@ FSTProcessor::analysis(FILE *input, FILE *output)
           fputwc_unlocked(val, output);
         }
       }
-      else if(last_incond)
-      {
-        printWord(sf.substr(0, sf.size()-input_buffer.diffPrevPos(last)),
-		  lf, output);
-        input_buffer.setPos(last);
-        input_buffer.back(1);
-      }
       else if(last_postblank)
       {
         printWord(sf.substr(0, sf.size()-input_buffer.diffPrevPos(last)),
@@ -618,6 +725,21 @@ FSTProcessor::analysis(FILE *input, FILE *output)
         input_buffer.setPos(last);
         input_buffer.back(1);
       }
+      else if(last_preblank)
+      {
+	fputwc_unlocked(L' ', output);
+        printWord(sf.substr(0, sf.size()-input_buffer.diffPrevPos(last)),
+		  lf, output);
+        input_buffer.setPos(last);
+        input_buffer.back(1);
+      }
+      else if(last_incond)
+      {
+        printWord(sf.substr(0, sf.size()-input_buffer.diffPrevPos(last)),
+		  lf, output);
+        input_buffer.setPos(last);
+        input_buffer.back(1);
+      }
       else if(isAlphabetic(val) && 
               ((sf.size()-input_buffer.diffPrevPos(last)) > lastBlank(sf) || 
                lf == L""))
@@ -628,19 +750,35 @@ FSTProcessor::analysis(FILE *input, FILE *output)
         }         
         while((val = readAnalysis(input)) && isAlphabetic(val));
 
-        unsigned int limit = sf.find(L' ');
+        unsigned int limit = firstNotAlpha(sf);
         unsigned int size = sf.size();
         limit = (limit == static_cast<unsigned int>(wstring::npos)?size:limit);
-        input_buffer.back(1+(size-limit));
-        printUnknownWord(sf.substr(0, limit), output);
+        if(limit == 0)
+        {
+          input_buffer.back(sf.size());
+          fputwc_unlocked(sf[0], output);          
+        }
+        else
+        { 
+          input_buffer.back(1+(size-limit));
+          printUnknownWord(sf.substr(0, limit), output);
+        }
       }
       else if(lf == L"")
       {
-        unsigned int limit = sf.find(L' ');
+        unsigned int limit = firstNotAlpha(sf);
         unsigned int size = sf.size();
         limit = (limit == static_cast<unsigned int >(wstring::npos)?size:limit);
-        input_buffer.back(1+(size-limit));
-        printUnknownWord(sf.substr(0, limit), output);
+        if(limit == 0)
+        {
+          input_buffer.back(sf.size());
+          fputwc_unlocked(sf[0], output);          
+        }
+        else
+        { 
+          input_buffer.back(1+(size-limit));
+          printUnknownWord(sf.substr(0, limit), output);
+        }
       }
       else
       {
@@ -655,6 +793,7 @@ FSTProcessor::analysis(FILE *input, FILE *output)
       sf = L"";
       last_incond = false;
       last_postblank = false;
+      last_preblank = false;
     }
   }
   
@@ -662,9 +801,236 @@ FSTProcessor::analysis(FILE *input, FILE *output)
   flushBlanks(output);
 }
 
+void
+FSTProcessor::analysis_wrapper_null_flush(FILE *input, FILE *output)
+{
+  setNullFlush(false);
+  while(!feof(input)) 
+  {
+    analysis(input, output);
+    fputwc_unlocked(L'\0', output);
+    int code = fflush(output);
+    if(code != 0) 
+    {
+        wcerr << L"Could not flush output " << errno << endl;
+    }
+  }
+}
+
+void
+FSTProcessor::generation_wrapper_null_flush(FILE *input, FILE *output, 
+                                            GenerationMode mode)
+{
+  setNullFlush(false);
+  while(!feof(input)) 
+  {
+    generation(input, output, mode);
+    fputwc_unlocked(L'\0', output);
+    int code = fflush(output);
+    if(code != 0) 
+    {
+        wcerr << L"Could not flush output " << errno << endl;
+    }
+  }
+}
+
+void
+FSTProcessor::postgeneration_wrapper_null_flush(FILE *input, FILE *output)
+{
+  setNullFlush(false);
+  while(!feof(input)) 
+  {
+    postgeneration(input, output);
+    fputwc_unlocked(L'\0', output);
+    int code = fflush(output);
+    if(code != 0) 
+    {
+        wcerr << L"Could not flush output " << errno << endl;
+    }
+  }
+}
+
+void
+FSTProcessor::transliteration_wrapper_null_flush(FILE *input, FILE *output)
+{
+  setNullFlush(false);
+  while(!feof(input)) 
+  {
+    transliteration(input, output);
+    fputwc_unlocked(L'\0', output);
+    int code = fflush(output);
+    if(code != 0) 
+    {
+        wcerr << L"Could not flush output " << errno << endl;
+    }
+  }
+}
+
+void
+FSTProcessor::tm_analysis(FILE *input, FILE *output)
+{
+  State current_state = initial_state;
+  wstring lf = L"";
+  wstring sf = L"";
+  int last = 0;
+
+  while(wchar_t val = readTMAnalysis(input))
+  {
+    // test for final states
+    if(current_state.isFinal(all_finals))
+    {
+      if(iswpunct(val))
+      {
+        lf = current_state.filterFinalsTM(all_finals, alphabet, 
+					  escaped_chars,
+                                          blankqueue, numbers).substr(1);
+        last = input_buffer.getPos();
+        numbers.clear();
+      }
+    }
+    else if(sf == L"" && iswspace(val)) 
+    {
+      lf.append(sf);
+      last = input_buffer.getPos();
+    }
+
+    if(!iswupper(val))
+    {
+      current_state.step(val);
+    }
+    else
+    {
+      current_state.step(val, towlower(val));
+    }
+      
+    if(current_state.size() != 0)
+    {
+      if(val == -1)
+      {
+        sf.append(numbers[numbers.size()-1]);
+      }
+      else if(isLastBlankTM && val == L' ')
+      {
+        sf.append(blankqueue.back());
+      }
+      else
+      {
+        alphabet.getSymbol(sf, val);
+      }
+    }
+    else
+    {
+      if((iswspace(val) || iswpunct(val)) && sf == L"")
+      {
+        if(iswspace(val))
+        {
+          printSpace(val, output);
+        }
+        else
+        {
+          if(isEscaped(val))
+          {
+            fputwc_unlocked(L'\\', output);
+          }
+          fputwc_unlocked(val, output);
+        }
+      }
+      else if(!iswspace(val) && !iswpunct(val) && 
+              ((sf.size()-input_buffer.diffPrevPos(last)) > lastBlank(sf) || 
+               lf == L""))
+      {
+      
+        do
+        {
+          if(val == -1)
+          {
+            sf.append(numbers[numbers.size()-1]);
+          }
+          else if(isLastBlankTM && val == L' ')
+          {
+            sf.append(blankqueue.back());
+          }
+          else
+          {
+            alphabet.getSymbol(sf, val);
+          }
+        }         
+        while((val = readTMAnalysis(input)) && !iswspace(val) && !iswpunct(val));
+
+        if(val == 0)
+        {
+          fputws_unlocked(sf.c_str(), output);
+          return;
+        }
+
+        input_buffer.back(1);
+        fputws_unlocked(sf.c_str(), output);
+
+        while(blankqueue.size() > 0)
+        {
+          if(blankqueue.size() == 1 && isLastBlankTM)
+          {
+            break;
+          }
+          blankqueue.pop();
+        }        
+
+/*
+        unsigned int limit = sf.find(L' ');
+        unsigned int size = sf.size();
+        limit = (limit == static_cast<unsigned int>(wstring::npos)?size:limit);
+        input_buffer.back(1+(size-limit));
+        fputws_unlocked(sf.substr(0, limit).c_str(), output);
+*/      }
+      else if(lf == L"")
+      {
+/*        unsigned int limit = sf.find(L' ');
+        unsigned int size = sf.size();
+        limit = (limit == static_cast<unsigned int >(wstring::npos)?size:limit);
+        input_buffer.back(1+(size-limit));
+        fputws_unlocked(sf.substr(0, limit).c_str(), output);
+*/
+        input_buffer.back(1);
+        fputws_unlocked(sf.c_str(), output);
+
+        while(blankqueue.size() > 0)
+        {
+          if(blankqueue.size() == 1 && isLastBlankTM)
+          {
+            break;
+          }
+          blankqueue.pop();
+        }        
+
+      }
+      else
+      {
+        fputwc_unlocked(L'[', output);
+        fputws_unlocked(lf.c_str(), output);
+        fputwc_unlocked(L']', output);
+        input_buffer.setPos(last);
+        input_buffer.back(1);
+      }
+	
+      current_state = initial_state;
+      lf = L"";
+      sf = L"";
+    }
+  }
+  
+  // print remaining blanks
+  flushBlanks(output);
+}
+
+
 void
 FSTProcessor::generation(FILE *input, FILE *output, GenerationMode mode)
 {
+  if(getNullFlush())
+  {
+    generation_wrapper_null_flush(input, output, mode);
+  }
+
   State current_state = initial_state;
   wstring sf = L"";
  
@@ -672,11 +1038,18 @@ FSTProcessor::generation(FILE *input, FILE *output, GenerationMode mode)
  
   skipUntil(input, output, L'^');
   int val;
+
   while((val = readGeneration(input, output)) != 0x7fffffff)
   {
-    if(val == L'$')
+    if(sf == L"" && val == L'=')
     {
-      if(sf[0] == L'*')
+      fputwc(L'=', output);
+      val = readGeneration(input, output);
+    }
+    
+    if(val == L'$' && outOfWord)
+    {
+      if(sf[0] == L'*' || sf[0] == L'%')
       {
 	if(mode != gm_clean)
         {
@@ -737,7 +1110,7 @@ FSTProcessor::generation(FILE *input, FILE *output, GenerationMode mode)
     {
       // do nothing
     }
-    else if(sf.size() > 0 && sf[0] == L'*')
+    else if(sf.size() > 0 && (sf[0] == L'*' || sf[0] == L'%' ))
     {
       alphabet.getSymbol(sf, val);
     }
@@ -762,6 +1135,11 @@ FSTProcessor::generation(FILE *input, FILE *output, GenerationMode mode)
 void
 FSTProcessor::postgeneration(FILE *input, FILE *output)
 {
+  if(getNullFlush())
+  {
+    postgeneration_wrapper_null_flush(input, output);
+  }
+
   bool skip_mode = true;
   State current_state = initial_state;
   wstring lf = L"";
@@ -923,6 +1301,11 @@ FSTProcessor::postgeneration(FILE *input, FILE *output)
 void
 FSTProcessor::transliteration(FILE *input, FILE *output)
 {
+  if(getNullFlush())
+  {
+    transliteration_wrapper_null_flush(input, output);
+  }
+
   State current_state = initial_state;
   wstring lf = L"";
   wstring sf = L"";
@@ -1014,6 +1397,7 @@ FSTProcessor::biltrans(wstring const &input_word, bool with_delim)
   unsigned int start_point = 1;
   unsigned int end_point = input_word.size()-2;
   wstring queue = L"";
+  bool mark = false;
   
   if(with_delim == false)
   {
@@ -1026,6 +1410,12 @@ FSTProcessor::biltrans(wstring const &input_word, bool with_delim)
     return input_word;
   }
   
+  if(input_word[start_point] == L'=')
+  {
+    start_point++;
+    mark = true;
+  }
+  
   bool firstupper = iswupper(input_word[start_point]);
   bool uppercase = firstupper && iswupper(input_word[start_point+1]);
 
@@ -1075,17 +1465,31 @@ FSTProcessor::biltrans(wstring const &input_word, bool with_delim)
                                          uppercase, firstupper, 0);
       if(with_delim)
       {      
-        result[0] = L'^';
+        if(mark)
+        {
+          result = L"^="+result.substr(1);
+        }
+        else
+        {
+          result[0] = L'^';
+        }
       }
       else
       {
-        result = result.substr(1);
+        if(mark)
+        {
+          result = L"=" + result.substr(1);
+        }
+        else
+        {
+          result = result.substr(1);
+        }
       }
     }
     
     if(current_state.size() == 0)
     { 
-      if(symbol != L"")
+      if(symbol != L"" && result != L"")
       {
         queue.append(symbol);
       }
@@ -1156,6 +1560,7 @@ FSTProcessor::biltransWithQueue(wstring const &input_word, bool with_delim)
   unsigned int start_point = 1;
   unsigned int end_point = input_word.size()-2;
   wstring queue = L"";
+  bool mark = false;
   
   if(with_delim == false)
   {
@@ -1168,12 +1573,18 @@ FSTProcessor::biltransWithQueue(wstring const &input_word, bool with_delim)
     return pair<wstring, int>(input_word, 0);
   }
   
+  if(input_word[start_point] == L'=')
+  {
+    start_point++;
+    mark = true;
+  }
+
   bool firstupper = iswupper(input_word[start_point]);
   bool uppercase = firstupper && iswupper(input_word[start_point+1]);
-
+  
   for(unsigned int i = start_point; i <= end_point; i++)
   {
-    int val;
+    int val = 0;
     wstring symbol = L"";
  
     if(input_word[i] == L'\\')
@@ -1216,18 +1627,32 @@ FSTProcessor::biltransWithQueue(wstring const &input_word, bool with_delim)
                                          escaped_chars,
                                          uppercase, firstupper, 0);
       if(with_delim)
-      {      
-        result[0] = L'^';
+      {
+        if(mark)
+        {
+          result = L"^=" + result.substr(1);
+        }
+        else
+        {      
+          result[0] = L'^';
+        }
       }
       else
       {
-        result = result.substr(1);
+        if(mark)
+        {
+          result = L"=" + result.substr(1);
+        }
+        else
+        {
+          result = result.substr(1);
+        }
       }
     }
     
     if(current_state.size() == 0)
     { 
-      if(symbol != L"")
+      if(symbol != L"" && result != L"")
       {
         queue.append(symbol);
       }
@@ -1297,6 +1722,7 @@ FSTProcessor::biltransWithoutQueue(wstring const &input_word, bool with_delim)
   wstring result = L"";
   unsigned int start_point = 1;
   unsigned int end_point = input_word.size()-2;
+  bool mark = false;
   
   if(with_delim == false)
   {
@@ -1309,6 +1735,12 @@ FSTProcessor::biltransWithoutQueue(wstring const &input_word, bool with_delim)
     return input_word;
   }
   
+  if(input_word[start_point] == L'=')
+  {
+    start_point++;
+    mark = true;
+  }
+
   bool firstupper = iswupper(input_word[start_point]);
   bool uppercase = firstupper && iswupper(input_word[start_point+1]);
 
@@ -1358,11 +1790,25 @@ FSTProcessor::biltransWithoutQueue(wstring const &input_word, bool with_delim)
                                          uppercase, firstupper, 0);
       if(with_delim)
       {      
-        result[0] = L'^';
+        if(mark)
+        {
+          result = L"^=" + result.substr(1);
+        }
+        else
+        {
+          result[0] = L'^';
+        }
       }
       else
       {
-        result = result.substr(1);
+        if(mark)
+        {
+          result = L"=" + result.substr(1);
+        }
+        else
+        {
+          result = result.substr(1);
+        }
       }
     }
     
@@ -1598,7 +2044,7 @@ FSTProcessor::SAO(FILE *input, FILE *output)
         }         
         while((val = readSAO(input)) && isAlphabetic(val));
 
-        unsigned int limit = sf.find(L' ');
+        unsigned int limit = firstNotAlpha(sf);
         unsigned int size = sf.size();
         limit = (limit == static_cast<unsigned int>(wstring::npos)?size:limit);
         input_buffer.back(1+(size-limit));
@@ -1608,7 +2054,7 @@ FSTProcessor::SAO(FILE *input, FILE *output)
       }
       else if(lf == L"")
       {
-        unsigned int limit = sf.find(L' ');
+        unsigned int limit = firstNotAlpha(sf);
         unsigned int size = sf.size();
         limit = (limit == static_cast<unsigned int>(wstring::npos)?size:limit);
         input_buffer.back(1+(size-limit));
@@ -1654,3 +2100,29 @@ FSTProcessor::setCaseSensitiveMode(bool const value)
 {
   caseSensitive = value;
 }
+
+void
+FSTProcessor::setNullFlush(bool const value)
+{
+  nullFlush = value;
+}
+
+bool
+FSTProcessor::getNullFlush()
+{
+  return nullFlush;
+}
+
+size_t
+FSTProcessor::firstNotAlpha(wstring const &sf)
+{
+  for(size_t i = 0, limit = sf.size(); i < limit; i++)
+  {
+    if(!isAlphabetic(sf[i]))
+    {
+      return i;
+    }   
+  }
+  
+  return wstring::npos;
+}
