@@ -51,11 +51,7 @@ XMLParseUtil::latin1(xmlChar const *input)
   int outputlen = xmlStrlen(input) + 1;
   int inputlen = xmlStrlen(input);
 
-#ifdef WIN32
-  unsigned char* output = (unsigned char*) alloca(outputlen);
-#else
-  unsigned char output[outputlen];
-#endif
+  unsigned char* output = new unsigned char[outputlen];
   
   if(UTF8Toisolat1(output, &outputlen, input, &inputlen) != 0)
   {
@@ -63,6 +59,7 @@ XMLParseUtil::latin1(xmlChar const *input)
 
   output[outputlen] = 0;
   string result = reinterpret_cast<char *>(output);
+  delete[] output;
   return result;  
 }
 
@@ -119,12 +116,10 @@ XMLParseUtil::towstring(xmlChar const * input)
 wstring 
 XMLParseUtil::stows(string const &str)
 {
-#ifdef WIN32
-  wchar_t* result = (wchar_t*) alloca(str.size());  
-#else
-  wchar_t result[str.size()];
-#endif
-  mbstowcs(result, str.c_str(), str.size());
+  wchar_t* result = new wchar_t[str.size()+1];
+  size_t retval = mbstowcs(result, str.c_str(), str.size());
+  result[retval] = L'\0';
   wstring result2 = result;
+  delete[] result;
   return result2;
 }
